
source card_img.tcl
set app(name)    "A Blackjack Testpad"
set app(version) "0.3"
set app(date)    "Jan 21st 2014"
set app(author)  "Lutz Wegner"

proc chkExit {} {
   global app data
   if { [tk_messageBox -icon question -title $app(name) -type yesno \
             -message "Do you really want to exit?"] == "yes" } {
        exit;
      }

  };# chkExit
  
proc main {} {
   global app data

   menu .m -tearoff 0
   . configure -menu .m
   menu .m.file -tearoff 0
   menu .m.help -tearoff 0
   .m add cascade -label "File" -menu .m.file -underline 0
   #.m add command -label "Deal!" -command {dealDeck} -underline 0
   .m add cascade -label "Help" -menu .m.help -underline 0
   .m.file add command -label "New Game" -command newGame \
                       -accelerator "F2" -underline 0
   #.m.file add command -label "Restart This Game" -command "newGame 1" \
                       -accelerator "F5" -underline 0
   .m.file add separator
   .m.file add command -label "Exit"          -command exit -underline 1
   .m.help add command -label "Rules"         -command showRules -underline 0 -accelerator "F1"
   .m.help add separator
   .m.help add command -label "About..."      -command aboutThisGame -underline 0
   
    #initialize 
	set data(suits) "s h d c"
	set data(ranks) "a k q j t 9 8 7 6 5 4 3 2"
	set data(values) "11 10 10 10 10 9 8 7 6 5 4 3 2"
	set data(allcards) {}
	foreach suit $data(suits) {
		foreach rank $data(ranks) {
			lappend data(allcards) "$rank$suit"
		}
	}
	set data(handbank) {}
	set data(handplayer) {}
	
	set data(posxbank) 150; set data(posybank) 50
	set data(posxplayer) 150; set data(posyplayer) 250
	set data(posxstack) 305
	set data(posystack) 150

	canvas .c -bg darkgreen -borderwidth 0 -highlightthickness 0 \
		-width 400 -height 400
	pack .c
	.c create text 200 30 -text "Bank" -fill white -font "Arial 16"
	.c create text 200 370 -text "Spieler" -fill white -font "Arial 16"
	
	.c create text 20 200 -text "Spiel beginnt" -fill white -font "Arial 14" \
		-tag "whowins" -anchor sw
	update

   bind . <F1>            {showRules}
   bind . <F2>            {newGame}
   
   pack .c -expand 1 -fill both
  #wm geometry . 800x550
   wm geometry . 400x400
   wm title . "$app(name) Version $app(version)"
   wm protocol . WM_DELETE_WINDOW chkExit
   
	set data(deck) [shuffleList $data(allcards)]
	focus -force .c
};# main


  proc aboutThisGame {} {
   global app

   set text "$app(name) is a strongly reduced version of 'Blackjack' written in Tcl/Tk. Some code (and card images) were taken from the game 'Solitaire' by Mike Griffiths, who in turn used code from 'Once in a Lifetime', by Jeff Godrey, who in turn .... The main purpose is to illustrate moving and showing card images in Tcl/Tk."
   tk_messageBox -icon info -title "About $app(name) Version $app(version)" \
                 -message $text

  };# aboutThisGame
  
 proc showRules {} {
	global app

	set w .rules
	if { [winfo exists $w] } {
        wm deiconify $w
        raise $w
        focus $w
        return;
      }
	toplevel $w
	wm title $w "$app(name) - Rules"
	frame $w.f
	text $w.f.t -wrap word
	scrollbar $w.f.sb
	$w.f.t  configure -yscrollcommand "$w.f.sb set"
	$w.f.sb configure -command "$w.f.t yview"
	$w.f.t tag configure head -font {{} 20 bold} -justify center
	$w.f.t tag configure sub -font {{} 16 italic} -justify center
	$w.f.t insert end "$app(name)\n" head
	$w.f.t insert end "Version $app(version) - Regeln\n\n" sub
	$w.f.t insert end "In dieser stark vereinfachten Version von Black Jack spielt die Bank gegen genau einen Spieler. Die Bank erhlt zwei Karten, wovon die zweite verdeckt bleibt. Der Spieler erhlt offen zwei Karten. Er kann jetzt weitere Karten anfordern bis er 'Keine Karte' anklickt.\n"
	$w.f.t insert end "Dann wird die zweite Karte der Bank aufgedeckt und die Bank muss eine dritte Karte nehmen, wenn die Punktzahl unter 17 liegt, sonst nicht. Dass Ass zhlt hier immer 11 Punkte.\n"
	$w.f.t insert end "Gewonnen hat, wer mehr Punkte hat, allerdings nicht mehr als 21. Haben Bank und Spieler gleiche Punktzahl unter 21, gewinnt die Bank. Nur wenn Bank und Spieler beide ber 21 liegen, endet das Spiel unentschieden."
	
	$w.f.t configure -state disabled
	pack $w.f    -side top   -expand 1 -fill both
	pack $w.f.t  -side left  -expand 1 -fill both
	pack $w.f.sb -side right -fill y
	frame  $w.f2
	pack   $w.f2 -side top -pady 8
	button $w.f2.b -text "Close" -command "wm withdraw $w" -width 9
	pack   $w.f2.b
	wm protocol $w WM_DELETE_WINDOW "wm withdraw $w"
	focus $w.f.t

  };# showRules


proc shuffleList { list } {
	set n [llength $list]
	for { set i 0 } { $i < $n } { incr i } {
		set j [expr { int( rand() * $n ) }]
		set temp [lindex $list $i]
		lset list $i [lindex $list $j]
		lset list $j $temp
       }
	return $list;
};# shuffleList

proc turnOver { card } {
global data
	set img ::img::$card
	set xy [.c coords "tag$card"]
	.c delete "tag$card"
	.c create image [lindex $xy 0] [lindex $xy 1] \
		-image $img -anchor nw -tags "tag$card"
}

proc dealBank { } {
global data
	set card [lindex $data(deck) 0]
	lappend data(handbank) $card
	set len [llength $data(handbank)]
	lremove data(deck) $card
	.c raise "tag$card"
	moveCard .c "tag$card" [expr {$data(posxbank) + ($len - 1) * 20}] \
		$data(posybank) 10
	if {$len != 2} {
		#second card remains unturned
		turnOver $card	
	}
}

proc dealPlayer { } {
global data
	set sy $data(posyplayer)
	set sx $data(posxplayer)
	set card [lindex $data(deck) 0]
	lappend data(handplayer) $card
	set len [llength $data(handplayer)]
	lremove data(deck) $card
	.c raise "tag$card"
	moveCard .c "tag$card" [expr {$data(posxplayer) + ($len - 1) * 20}] \
		$data(posyplayer) 10
	turnOver $card	
	if {[llength $data(handplayer)] == 2} {
		button .c.bk -text "Karte" -padx 5 -pady 5 \
			-width 11 -command "dealPlayer"
		place .c.bk -x 20 -y 250
		button .c.bn -text "Keine Karte" -padx 5 -pady 5 \
			-width 11 -command "checkWin"
		place .c.bn -x 20 -y 310
	}
	set vp [getValue $data(handplayer)]
	.c itemconfigure "whowins" -text "Spieler hlt $vp"
	if {$vp > 21} {checkWin}
}

proc getValue { hand } {
global data
	set v 0
	foreach card $hand {
		set v [expr $v + [lindex $data(values) \
			[lsearch -exact $data(ranks) [string index $card 0]]]]
	}
	return $v
}

proc checkWin {} {
global data
	destroy .c.bk .c.bn
	after 100
	turnOver [lindex $data(handbank) 1]
	set vp [getValue $data(handplayer)]
	set vb [getValue $data(handbank)]
	if {$vb < 17} {
		dealBank ; #Bank muss Karte aufnehmen bei weniger als 17
	}
	set vb [getValue $data(handbank)]
	if {$vp > 21 && $vb > 21} {
		.c itemconfigure "whowins" -text "Spieler und Bank ber 21, \nunentschieden."
		return
	}
	if {$vb > 21} {
		.c itemconfigure "whowins" -text "Bank ber 21, \nder Spieler gewinnt."
		return
	} elseif {$vp > 21} {
		.c itemconfigure "whowins" -text "Spieler ber 21, \ndie Bank gewinnt."
	} elseif { $vb >= $vp } {
		.c itemconfigure "whowins" -text "Bank $vb, Spieler $vp, \ndie Bank gewinnt."
	} else {
		.c itemconfigure "whowins" -text "Bank $vb, Spieler $vp, \nder Spieler gewinnt."
	}
}

proc moveCard {c item tox toy {steps 10}} {
# modified from solitaire.tcl
   set orig [$c coords $item];
   set origx [lindex $orig 0]; set origy [lindex $orig 1]
   set diffx [expr {$tox -$origx}]
   set diffy [expr {$toy - $origy}]
   set stepx [expr {$diffx / $steps}]
   set stepy [expr {$diffy / $steps}]
   for {set i 1} {$i < $steps} {incr i} {
    $c move $item $stepx $stepy
     update idletasks
     after 50 
	}
	$c coords $item $tox $toy
  };# moveCard

proc lremove {listVariable value} {
    upvar $listVariable var
    set idx [lsearch -exact $var $value]
    set var [lreplace $var $idx $idx]
}

proc clearGame {} {
   global data
	#delete remaining card images
	destroy .c.bk .c.bn
   foreach x $data(allcards) {
		.c delete "tag$x"
	}
	set data(handbank) {}
	set data(handplayer) {}
	.c itemconfigure "whowins" -text "Neues Spiel"
  };# clearGame

proc newGame {} {
   global data
   clearGame
   foreach card $data(allcards) {
	#.c bind "tag$card" <ButtonPress-1>   [list playTrick %x %y]
	set id [.c create image $data(posxstack) $data(posystack) \
                         -image "::img::back" -anchor nw -tags "tag$card"]
	} 
	set data(deck) [shuffleList $data(allcards)]
	dealBank; dealBank; #zwei Karten
	dealPlayer; dealPlayer; #auch zwei Karten
	#showHands 
  };# newGame
 
 main; #initialisieren
 newGame
 