#demo for heapsort with illustrations
package require Tcl 8.5
focus -force .
set keycounter 0
set weiter false; set go false
set Xloc(0) 0; set Yloc(0) 0
set xoffset 100; set yoffset 50
canvas .c -width 800 -height 500 -bg white
pack .c
frame .f 
pack .f
button .f.start -text Start -command {set go true}
pack .f.start -padx 5 -pady 5 -side left
button .f.stop -text Stop -command {vwait weiter}
pack .f.stop -padx 5 -pady 5 -side left
button .f.continue -text Weiter -command {set weiter true}
pack .f.continue -padx 5 -pady 5 -side left
button .f.cancel -text Ende -command {exit}
pack .f.cancel -padx 5 -pady 5 -side left
label .f.anzeige1 -text "Schluesselvergleiche = $keycounter"
pack .f.anzeige1 -padx 5 -pady 5 -side left

set input {50 10 30 70 90 100 80 20 60 40}
set N [llength $input]

#Variante mit verbessertem Schluesselzaehler
proc heapsort {list {count ""}} {
	if {$count eq ""} {
		set count [llength $list]
	}
	for {set i [expr {$count/2 - 1}]} {$i >= 0} {incr i -1} {
		siftDown list $i [expr {$count - 1}]
	}
	for {set i [expr {$count - 1}]} {$i > 0} {incr i -1} {
		swap list $i 0
		showProgress $i [expr {$count - 1}] 
		siftDown list 0 [expr {$i - 1}]
	}
	showProgress 0 [expr $count - 1]
	return $list
}
proc siftDown {varName i j} {
upvar 1 $varName a
global keycounter
	while true {
		set child [expr {$i*2 + 1}]
		if {$child > $j} {
			break
		}
		#incr keycounter
		if {$child+1 <= $j} {
			incr keycounter
			if {[lindex $a $child] < [lindex $a $child+1]} {
				incr child
			}
		}
		incr keycounter
		if {[lindex $a $i] >= [lindex $a $child]} {
			break
		}
		swap a $i $child
		set i $child
	}
}

proc showProgress {from to} {
global xoffset yoffset
  .c coords "progress" [expr $xoffset + $from *60] [expr $yoffset -10] \
		[expr $xoffset + 60 + $to *60] [expr $yoffset -10] 
}

proc swap {varName x y} {
upvar 1 $varName a
showSwapOn $x $y
after 1000
set tmp [lindex $a $x]
lset a $x [lindex $a $y]
lset a $y $tmp
showArray $a
after 1000
showSwapOff $x $y
}

#create the array as sequence of rectangles, say 60 x 60 with a slim border 
#and number inside. Start on left with xoffset and yoffset from top.
proc createArray {L} {
global xoffset yoffset
  set n [llength $L]
  set x $xoffset
  for {set i 0} {$i < $n} {incr i} {
    .c create rectangle $x  $yoffset [expr $x + 60] [expr $yoffset + 60] \
		-outline "red" -fill blue -tag "field-$i"
	.c create text [expr $x + 30] [expr $yoffset + 30] \
		-fill white -text [lindex $L $i] -font "Arial 18 bold" \
		-tag "number-$i"
	.c create line [expr $xoffset + $n *60] [expr $yoffset + 70] \
		[expr $xoffset + $n *60] [expr $yoffset + 70] \
		-width 6 -fill darkgreen -tag progress
	set x [expr $x + 60]
  }
}

proc showArray {L} {
	global keycounter
	set n [llength $L]
    for {set i 0} {$i < $n} {incr i} {
		.c itemconfigure "number-$i" -text "[lindex $L $i]"
	}
	.f.anzeige1 configure -text "Schluesselvergleiche = $keycounter"
	update
}

proc showSwapOn {i j} {
	.c itemconfigure "field-$i" -fill "red"
	.c itemconfigure "field-$j" -fill "red"
	update
}

proc showSwapOff {i j} {
	.c itemconfigure "field-$i" -fill "blue"
	.c itemconfigure "field-$j" -fill "blue"
	update
}

proc compute_node_locs {n} {
#returns a list of [x, y] node locations for a heap with n nodes
global Xloc Yloc xoffset yoffset
	set w [expr [.c cget -width] - 2 * $xoffset]
	set y [expr {$yoffset + 100}]
	set i 0; set j 1
	while {$i < $n} {
		set b [expr $w/$j]
		for {set k 0} {$k < $j} {incr k} {
			set Xloc($i) [expr $b / 2 + $b*$k + $xoffset] 
			set Yloc($i) $y
			incr i
		}
		set j [expr 2 * $j]
		set y [expr $y + 100]
	}
}

proc drawTree {a} {
global Xloc Yloc
	set n [llength $a]
	for {set i 0} {$i < $n} {incr i} {
		set j [expr $i + 1]
		if {2 * $j - 1 < $n} {
			.c create line $Xloc($i) $Yloc($i) \
				$Xloc([expr {2*$j - 1}]) $Yloc([expr {2*$j - 1}]) \
				-width 2
		}
		if {2*$j < $n} {
			.c create line $Xloc($i) $Yloc($i) \
				$Xloc([expr {2*$j}]) $Yloc([expr {2*$j}]) \
				-width 2
		}
		.c create oval [expr {$Xloc($i) - 20}] [expr {$Yloc($i) - 20}] \
			[expr {$Xloc($i) + 20}] [expr {$Yloc($i)+20}] -fill blue \
			-tag "field-$i"
		.c create text $Xloc($i) $Yloc($i) -text [lindex $a $i] \
			-fill white -font "Arial 14 bold" -tag "number-$i"
	}
}

set rerun false
while {true} {
	createArray $input
	compute_node_locs $N
	drawTree $input
	if {!$rerun} {vwait go}
	set output [heapsort $input]
	after 1000
	showArray $output
	vwait go
	.c delete all; set keycounter 0; set rerun true
}